/*==============================================================================*/
/*  Copyright (C) 2020, Heartland Data inc. All Rights Reserved.                */
/*                                                                              */
/*  Title  :   Data Writing Driver                                              */
/*  EventID:   Via the driver                                                   */
/*  AppVer :   DT+ Ver1.00`                                                    */
/*  FileOD :   16b                                                              */
/*  Version:   1.0                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Header for File IO                                                  */
/*==============================================================================*/
#include <stdio.h>

/*==============================================================================*/
/*  Macro:  DT_UINT                                                             */
/*  Desc:   Please change Test Point argument type for DT+ Project setting.     */
/*==============================================================================*/
#define	DT_UINT unsigned int

/*==============================================================================*/
/*  Desc:   Test Point Parametaer                                               */
/*==============================================================================*/
#define DT_VARIABLE_BIT		0x2

/*==============================================================================*/
/*  Macro:  DT_ADD_EVENT_INFO                                                   */
/*  Desc:   Please set 1 when you add Event Value to Test Point.                */
/*==============================================================================*/
#define	DT_ADD_EVENT_INFO	0

static FILE *dat_file = NULL;
static int BusoutMode = 0;
#if DT_ADD_EVENT_INFO
static unsigned long long time = 0;
DWORD pid = 0;
#else
static unsigned int time = 0;
#endif

/*==============================================================================*/
/*  Macro:  BusOutModeSet                                                       */
/*  Desc:   0 => open file 1 => writing 2 => close file                         */
/*==============================================================================*/
#define BusOutModeSet(mode) BusoutMode = mode

/*==============================================================================*/
/*  Func:   _TP_BusOut                                                          */
/*  Desc:   Called by Test Point                                                */
/*==============================================================================*/
void _TP_BusOut( DT_UINT addr, DT_UINT dat )
{
	unsigned char buff[16] = {0};

	if (BusoutMode == 0) {
		/* Customize File Path */
		dat_file = fopen("ConvertData.DAT", "wb+");
		if (dat_file == NULL) {
			return;
		}
		BusOutModeSet(1);
	}

	/* time data(Customize) */
	time++;

#if DT_ADD_EVENT_INFO
	/* pid(Customize) */
	pid++;
#endif

	buff[0] = (unsigned char)((dat) & 0xFF);
	buff[1] = (unsigned char)((dat >> 8) & 0xFF);
	buff[2] = (unsigned char)((addr) & 0xFF);
	buff[3] = (unsigned char)((addr >> 8) & 0xFF);
	buff[4] = (unsigned char)((addr >> 16) & 0xFF);

	/* time data */
	buff[5] = (unsigned char)((time) & 0xFF);
	buff[6] = (unsigned char)((time >> 8) & 0xFF);
	buff[7] = (unsigned char)((time >> 16) & 0xFF);
	buff[8] = (unsigned char)((time >> 24) & 0xFF);
#if DT_ADD_EVENT_INFO
	/* event_id */
	buff[9] = (unsigned char)((pid) & 0xFF);
	buff[10] = (unsigned char)((pid >> 8) & 0xFF);
	buff[11] = (unsigned char)((pid >> 16) & 0xFF);
	buff[12] = (unsigned char)((pid >> 24) & 0xFF);
#else
	buff[9] = (unsigned char)((time >> 32) & 0xFF);
	buff[10] = (unsigned char)((time >> 40) & 0xFF);
	buff[11] = (unsigned char)((time >> 48) & 0xFF);
	buff[12] = (unsigned char)((time >> 56) & 0xFF);
#endif

	/* Reserve */
	buff[13] = 0x00;
   	buff[14] = 0x00;
   	buff[15] = 0x00;

	fwrite(buff, sizeof(char), 16, dat_file);
	if (BusoutMode == 2) {
		fclose(dat_file);
	}
}

/*==============================================================================*/
/*  Func:   _TP_MemoryOutput                                                    */
/*  Desc:   Called by Variable TestPoint                                        */
/*==============================================================================*/
void _TP_MemoryOutput( DT_UINT addr, DT_UINT dat, void *value, DT_UINT size )
{
	char buff[16];
	char *p = (char*)value;
	int i, j, packetNum, count;

	if (size > 256) size = 256;
	packetNum = (size + 7) / 8;

	_TP_BusOut((addr | DT_VARIABLE_BIT), dat);	/* Memory Dump Start */

	buff[0] = size;
	buff[1] = packetNum;
	buff[2] = 0x03;
	buff[12] = 0x00;
	buff[13] = 0x00;
	buff[14] = 0x00;
	buff[15] = 0x00;

	count = 0;
	for (i = 0; i < packetNum; i++) {
		buff[3] = i;
		for (j = 0; j < 8; j++, count++, p++) {
			buff[4+j] = (count < size)?*p:0x00;
		}
		fwrite(buff, sizeof(char), 16, dat_file);
	}
	if (BusoutMode == 2) {
		fclose(dat_file);
	}
}
